//
// Header definitions for the x86 routines for the HERMES library
// Copyright (c) 1998 Christian Nentwich (brn@eleet.mcb.at)
// This source code is licensed under the GNU LGPL
//
// Please refer to the file COPYING.LIB contained in the distribution for
// licensing conditions
//
#ifndef __HERMES_HEAD_X86__
#define __HERMES_HEAD_X86__


#include "HermConf.h"

#ifndef NO_ASSEMBLER

// If you can't stand IFDEFS, then close your eyes now, please :)

// Ok, we start with normal function definitions
#ifdef __cplusplus
extern "C" {
#endif


#if defined __GNUC__
void ConvertX86(HermesInterface *) __attribute__ ((regparm(1)));
void Clear32_x86(HermesClearInterface *) __attribute__ ((regparm(1)));
void Clear24_x86(HermesClearInterface *) __attribute__ ((regparm(1)));
void Clear16_x86(HermesClearInterface *) __attribute__ ((regparm(1)));
void Clear8_x86(HermesClearInterface *) __attribute__ ((regparm(1)));
void ConvertI8_SetLookup(void *lookup) __attribute__ ((regparm(1)));
#else
void ConvertX86(HermesInterface *);
void Clear32_x86(HermesClearInterface *);
void Clear24_x86(HermesClearInterface *);
void Clear16_x86(HermesClearInterface *);
void Clear8_x86(HermesClearInterface *);
void ConvertI8_SetLookup(void *lookup);
#endif // GNUC

void Convert32_32RGB888();
void Convert32_24RGB888();
void Convert32_24BGR888();
void Convert32_16RGB565();
void Convert32_16BGR565();
void Convert32_16RGB555();
void Convert32_16BGR555();
void Convert32_8RGB332();

void Convert16_32RGB888();
void Convert16_32BGR888();
void Convert16_32RGBA888();
void Convert16_32BGRA888();
void Convert16_24RGB888();
void Convert16_24BGR888();
void Convert16_16RGB565();
void Convert16_16BGR565();
void Convert16_16RGB555();
void Convert16_16BGR555();
void Convert16_8RGB332();

void ConvertI8_32();
void ConvertI8_24();
void ConvertI8_16();
void ConvertI8_INDEX8();

  
extern int32 Convert16_32RGB888_LUT_X86[512];
extern int32 Convert16_32BGR888_LUT_X86[512];
extern int32 Convert16_32RGBA888_LUT_X86[512];
extern int32 Convert16_32BGRA888_LUT_X86[512];
  
#ifdef __cplusplus
}
#endif




// Now fix up the ELF underscore problem

#if defined(__ELF__) && defined(__GNUC__)
  #ifdef __cplusplus
  extern "C" {
  #endif

  void ConvertX86(HermesInterface *) __attribute__ ((alias ("_ConvertX86"))) 
	 __attribute__ ((regparm(1)));
  void Clear32_x86(HermesClearInterface *) __attribute__ ((alias ("_Clear32_x86"))) __attribute__ ((regparm(1)));
  void Clear24_x86(HermesClearInterface *)  __attribute__ ((alias ("_Clear24_x86"))) __attribute__ ((regparm(1)));
  void Clear16_x86(HermesClearInterface *)  __attribute__ ((alias ("_Clear16_x86"))) __attribute__ ((regparm(1)));
  void Clear8_x86(HermesClearInterface *)  __attribute__ ((alias ("_Clear8_x86"))) __attribute__ ((regparm(1)));

  void Convert32_32RGB888() __attribute__ ((alias ("_Convert32_32RGB888")));
  void Convert32_24RGB888() __attribute__ ((alias ("_Convert32_24RGB888")));
  void Convert32_24BGR888() __attribute__ ((alias ("_Convert32_24BGR888")));
  void Convert32_16RGB565() __attribute__ ((alias ("_Convert32_16RGB565")));
  void Convert32_16BGR565() __attribute__ ((alias ("_Convert32_16BGR565")));
  void Convert32_16RGB555() __attribute__ ((alias ("_Convert32_16RGB555")));
  void Convert32_16BGR555() __attribute__ ((alias ("_Convert32_16BGR555")));
  void Convert32_8RGB332() __attribute__ ((alias ("_Convert32_8RGB332")));

  void Convert16_32RGB888() __attribute__ ((alias ("_Convert16_32RGB888")));
  void Convert16_32BGR888() __attribute__ ((alias ("_Convert16_32BGR888")));
  void Convert16_32RGBA888() __attribute__ ((alias ("_Convert16_32RGBA888")));
  void Convert16_32BGRA888() __attribute__ ((alias ("_Convert16_32BGRA888")));
  void Convert16_24RGB888() __attribute__ ((alias ("_Convert16_24RGB888")));
  void Convert16_24BGR888() __attribute__ ((alias ("_Convert16_24BGR888")));
  void Convert16_16RGB565() __attribute__ ((alias ("_Convert16_16RGB565")));
  void Convert16_16BGR565() __attribute__ ((alias ("_Convert16_16BGR565")));
  void Convert16_16RGB555() __attribute__ ((alias ("_Convert16_16RGB555")));
  void Convert16_16BGR555() __attribute__ ((alias ("_Convert16_16BGR555")));
  void Convert16_8RGB332() __attribute__ ((alias ("_Convert16_8RGB332")));
  
  void ConvertI8_32() __attribute__ ((alias ("_ConvertI8_32")));
  void ConvertI8_24() __attribute__ ((alias ("_ConvertI8_24")));
  void ConvertI8_16() __attribute__ ((alias ("_ConvertI8_16")));
  void ConvertI8_INDEX8() __attribute__ ((alias ("_ConvertI8_INDEX8")));
 
  void ConvertI8_SetLookup(void *lookup) __attribute__ ((alias ("_ConvertI8_SetLookup"))) __attribute__ ((regparm(1)));

  extern int32 Convert16_32RGB888_LUT_X86[512] __attribute__ ((alias ("_Convert16_32RGB888_LUT_X86")));
  extern int32 Convert16_32BGR888_LUT_X86[512]  __attribute__ ((alias ("_Convert16_32BGR888_LUT_X86")));
  extern int32 Convert16_32RGBA888_LUT_X86[512] __attribute__ ((alias ("_Convert16_32RGBA888_LUT_X86")));
  extern int32 Convert16_32BGRA888_LUT_X86[512] __attribute__ ((alias ("_Convert16_32BGRA888_LUT_X86")));
  
  #ifdef __cplusplus
  }
  #endif

#endif // ELF & GNU



// Make it run with WATCOM C (hopefully)
#ifdef __WATCOMC__
#pragma warning 601 9
#pragma aux ConvertX86 "_*" parm [EAX] modify [EAX EBX ECX EDX ESI EDI]
#pragma aux Clear32_x86 "_*" parm [EAX] modify [EAX EBX ECX EDX ESI EDI]
#pragma aux Clear24_x86 "_*" parm [EAX] modify [EAX EBX ECX EDX ESI EDI]
#pragma aux Clear16_x86 "_*" parm [EAX] modify [EAX EBX ECX EDX ESI EDI]
#pragma aux Clear8_x86 "_*" parm [EAX] modify [EAX EBX ECX EDX ESI EDI]

#pragma aux Convert32_32RGB888 "_*"
#pragma aux Convert32_24RGB888 "_*"
#pragma aux Convert32_24BGR888 "_*"
#pragma aux Convert32_16RGB565 "_*"
#pragma aux Convert32_16BGR565 "_*"
#pragma aux Convert32_16RGB555 "_*"
#pragma aux Convert32_16BGR555 "_*"
#pragma aux Convert32_8RGB332 "_*"

#pragma aux Convert16_32RGB888 "_*"
#pragma aux Convert16_32BGR888 "_*"
#pragma aux Convert16_32RGBA888 "_*"
#pragma aux Convert16_32BGRA888 "_*"
#pragma aux Convert16_24RGB888 "_*"
#pragma aux Convert16_24BGR888 "_*"
#pragma aux Convert16_16RGB565 "_*"
#pragma aux Convert16_16BGR565 "_*"
#pragma aux Convert16_16RGB555 "_*"
#pragma aux Convert16_16BGR555 "_*"
#pragma aux Convert16_8RGB332 "_*"

#pragma aux ConvertI8_32 "_*"
#pragma aux ConvertI8_24 "_*"
#pragma aux ConvertI8_16 "_*"
#pragma aux ConvertI8_INDEX8 "_*"

#pragma aux ConvertI8_SetLookup "_*" parm [EAX]
#endif // __WATCOMC__



#endif // NO_ASSEMBLER


#endif 
