//
// DirectX Primary class for OpenPTC 1.0 C++ Implementation
// Copyright (c) 1999 Glenn Fiedler (ptc@gaffer.org)
// This source code is licensed under the GNU LGPL
// See http://www.gnu.org/copyleft/lgpl.html for details
//

#ifndef __PTC_DIRECTX_PRIMARY_H
#define __PTC_DIRECTX_PRIMARY_H

// include files
#include <ddraw.h>
#include "Core/Area.h"
#include "Core/Copy.h"
#include "Core/Clear.h"
#include "Core/Types.h"
#include "Core/Config.h"
#include "Core/Format.h"
#include "Core/Palette.h"
#include "Win32/Window.h"




class DirectXPrimary
{
    public:

        // setup
        DirectXPrimary(Win32Window &window,LPDIRECTDRAW2 lpDD2,int pages,bool video,bool fullscreen);
        ~DirectXPrimary();

        // sychronization
        void update();

        // surface memory access
        void* lock();
        void unlock();

        // clear surface
        void clear();
        void clear(const Color &color,const Area &area);

        // surface palette
        void palette(const Palette &palette);
        const Palette& palette() const;

        // surface clip area
        void clip(const Area &area);

        // data access
        int width() const;
        int height() const;
        int pages() const;
        int pitch() const;
        const Area& area() const;
        const Area& clip() const;
        const Format& format() const;

        // directdraw data access
        LPDIRECTDRAWSURFACE lpDDS() const;
        LPDIRECTDRAWSURFACE2 lpDDS2() const;

        // primary management
        void activate();
        void deactivate();
        bool active() const;
        void block() const;
        void save() const;
        void restore() const;

    private:

        // pixel format translation
        Format translate(DDPIXELFORMAT const &ddpf);

        // close
        void close();

        // data
        int m_width;
        int m_height;
        int m_pages;
        Area m_area;
        Area m_clip;
        Format m_format;

        // active flag
        bool m_active;

        // clear object
        Clear m_clear;

        // window object
        Win32Window *m_window;

        // locked buffer
        void *m_locked;

        // fullscreen flag
        bool m_fullscreen;

        // palette object
        Palette m_palette;

        // ddraw interfaces
        LPDIRECTDRAW2 m_lpDD2;
        LPDIRECTDRAWSURFACE m_lpDDS;
        
        // ddraw working surfaces
        LPDIRECTDRAWSURFACE m_back;
        LPDIRECTDRAWSURFACE m_front;
};




#endif
